#include <Servo.h>
#include "SoftwareSerial.h"

SoftwareSerial softSerial(2, 3); // RX, TX

Servo motorE;  // crea el objecte motor esquerra
Servo motorD;  // crea el objecte motor dret
Servo motorCAP;

// Definim els diferents pins connectats a cada motor
#define pinMotorE   8
#define pinMotorD   10
#define pinMotorCAP 11

// Velocitat de transmissió posem una velocitat intermitja
const long baudRate = 19200;
// Emmagatzema l'ordre a gestionar pel robot
String ordre = ""; 

// Rutina principal, iniciem el sistema
void setup() {
  // Iniciem els ports serie
  Serial.begin(baudRate);     // Comunicació de debug
  softSerial.begin(baudRate); // Comunicació amb el mòdul ESP8266
  Serial.println("Robot iniciado"); // Indiquem que hem iniciat
  R2D2(); // Ho indiquem també per so

}
// Rutina d'execució
void loop() {
  // Mirem si ha cap ordre rebuda pel ESP8266
  if (softSerial.available()) {
    // Llegir del port
    char c = (char)softSerial.read();
    // Es pinta a pantalla per debug
    Serial.print(c);
    // Si hi ha un retorn de carro (13) s'ha rebut una ordre
    switch (c) {
      case 13:  gestiona(ordre);
        break;
      case 10:  break;
      // Si rebem un caràcter @ indica principi d'una ordre, iniciem la variable
      case '@': ordre = "";
        break;
      // Afegim a la variable els caràcters següents  
      default : ordre += c;
        break;
    }
  }
}

// Funció que gestiona les ordres rebudes
void gestiona(String ordre) {
  if (ordre.equals("ARRANCA")) {
    motorE.attach(pinMotorE);  // Associem els pins dels motors a la llibreria
    motorD.attach(pinMotorD);
    motorE.write(180);        // Màxima potència a cada motor però cadascun en un sentit
    motorD.write(0);
    return;
  }
  if (ordre.equals("PARA")) {
    motorE.detach();        // Per parar només cal desassociar
    motorD.detach();
    return;
  }
  if (ordre.equals("ENRERE")) {
    motorE.attach(pinMotorE); // Enrere és igual que ARRANCA però canvia el sentit de cada motor
    motorE.write(0);
    motorD.attach(pinMotorD);
    motorD.write(180);
    return;
  }
  if (ordre.equals("DRETA")) {
    motorE.detach();           // Per anar cap a la dreta es deixa parat el motor dret
    motorD.attach(pinMotorD);  // com el de l'esquerra avança, el robot pivota sobre la roda dreta
    motorD.write(0);
    return;
  }
  if (ordre.equals("ESQUERRA")) {
    motorE.attach(pinMotorE);  // Igual que a la dreta però detenim la roda esquerra
    motorE.write(180);
    motorD.detach();
    return;
  }
  if (ordre.equals("VOLTA_D")) {

    motorE.attach(pinMotorE);  // Per donar voltes els dos motors van en el mateix sentit
    motorD.attach(pinMotorD);
    motorE.write(0);
    motorD.write(0);
    return;
  }
  if (ordre.equals("VOLTA_E")) {
    motorE.attach(pinMotorE);
    motorD.attach(pinMotorD);
    motorE.write(180);
    motorD.write(180);
    return;
  }
  if (ordre.equals("NO_NO")) {
    motorCAP.attach(pinMotorCAP);    // Fem un bucle que varia la posició del motor del CAP
    for (int i = 0; i < 3 ; i++) {
      motorCAP.write(180);           // 180º
      delay(300);
      motorCAP.write(0);             // 0º Just a l'altre banda
      delay(300);
    }
    motorCAP.detach();              // Parem el motor
    R2D2();                         // Emete'm també un so
    return;
  }
  if (ordre.equals("SILBA")) {
    wolfWhistle();                  // So que emula que el robot xiscla
  }
  if (ordre.equals("SOUND")) {
    starWars();                     // Marxa imperial d'Star Wars
  }
}
