/*-------------------------------------------------------------------*
 |            P R O J E C T E     M y R o b o t                      |
 |-------------------------------------------------------------------|
 |  Control per veu i pantalla d'un robot fabricat amb un Arduino    |
 |-------------------------------------------------------------------|
 |                       (C) Pedro Puertas 2019                      |
 *------------------------------------------------------------------*/
package myrobot.main;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Vibrator;
import android.speech.RecognitionListener;
import android.speech.RecognizerIntent;
import android.speech.SpeechRecognizer;
import android.support.v4.app.ActivityCompat;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.Toast;

import java.util.ArrayList;

import myrobot.main.comm.Callback;
import myrobot.main.comm.Communicator;

@SuppressWarnings("JavaDoc")
public class MainActivity extends Activity implements RecognitionListener, View.OnTouchListener, Callback {

    private final int PERMISSION_RECORD_AUDIO = 100;

    // Reconeixement de veu
    private SpeechRecognizer mSpeechRecognizer;
    private Intent mSpeechRecognizerIntent;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Forçar que no canvïi d'oriebtació
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_NOSENSOR);

        // Mostrar finestra de Splash
        splash();

        // Consultem la IP del robot a la configuració
        SharedPreferences prop = getApplicationContext().getSharedPreferences("settings.properties", MODE_PRIVATE);
        Communicator.robotIP = prop.getString("robotIP", "");

        // Sol·licitar permissos per accedir al micròfon
        ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.RECORD_AUDIO}, PERMISSION_RECORD_AUDIO);

        // Iniciar el reconeixement de veu
        mSpeechRecognizer = SpeechRecognizer.createSpeechRecognizer(this);
        mSpeechRecognizer.setRecognitionListener(this);
        mSpeechRecognizerIntent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);
        mSpeechRecognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM);
        mSpeechRecognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName());

        // Assignar funcionalitat a cada botó
        ImageView btn = findViewById(R.id.btnSpeak);
        btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mSpeechRecognizer.startListening(mSpeechRecognizerIntent);
            }
        });
        btn.setOnTouchListener(this);
        // Assignem a cada botó un gestor de "touch" que farà animació quan es premi
        btn = findViewById(R.id.btnUP);
        btn.setOnTouchListener(this);
        btn = findViewById(R.id.btnDOWN);
        btn.setOnTouchListener(this);
        btn = findViewById(R.id.btnLEFT);
        btn.setOnTouchListener(this);
        btn = findViewById(R.id.btnRIGHT);
        btn.setOnTouchListener(this);
        btn = findViewById(R.id.btnSTOP);
        btn.setOnTouchListener(this);
        btn = findViewById(R.id.btnNO);
        btn.setOnTouchListener(this);
        btn = findViewById(R.id.btnSOUND);
        btn.setOnTouchListener(this);
        btn = findViewById(R.id.btnLeftLoop);
        btn.setOnTouchListener(this);
        btn = findViewById(R.id.btnRightLoop);
        btn.setOnTouchListener(this);
    }

    /**
     * Funció que crea les opcions de menú contextual
     */
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        return true;
    }

    /**
     * Funció de control de quan es selecciona una opció del menú
     *
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
            case R.id.settings:
                goSettings(null);
                return true;
            case R.id.acercaDe:
                goAcercaDe(null);
                return true;
        }
        return false;
    }

    /**
     * Funció que fa vibrar la pantalla i canvia el botó del color pitjat
     *
     * @param v
     * @param event
     * @return
     */
    public boolean onTouch(View v, MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            v.setAlpha(0.2f);
        } else {
            v.setAlpha(1f);
            Vibrator vibe = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);
            if (vibe != null) vibe.vibrate(100);
        }
        return false;
    }

    /**
     * Demanar els permisos d'accés al micròfon
     *
     * @param requestCode
     * @param permissions
     * @param grantResults
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case PERMISSION_RECORD_AUDIO:
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    // Permissos aconseguits
                    Log.d(MainActivity.class.getSimpleName(), "Permisos MICRO OK");
                }
                break;
        }
    }

    /*
       Funcions diverses del reconeixement que no cal implementar
     */
    public void onBeginningOfSpeech() {
    }

    public void onBufferReceived(byte[] buffer) {
    }

    public void onEndOfSpeech() {
    }

    public void onError(int error) {
    }

    public void onEvent(int eventType, Bundle params) {
    }

    public void onPartialResults(Bundle partialResults) {
    }

    public void onReadyForSpeech(Bundle params) {
    }

    public void onRmsChanged(float rmsdB) {
    }

    /**
     * Funció que analitza els resultats i controla el robot amb la veu, en aquest punt es podria treballar tota la
     * lògica per fer el robot més intel·ligent, de fet el que es fa es donar la "intel·ligencia" des del telèfon
     *
     * @param results
     */
    public void onResults(Bundle results) {
        ArrayList<String> matches = results.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION);
        /*
        if (matches.contains("Como te llamas")) {
            Toast.makeText(getBaseContext(), "Recognision OK!!!", Toast.LENGTH_SHORT).show();
            return;
        }
        */
        if (matches == null) return;

        if (matches.contains("como te llamas")) {
            Communicator.sendToRobot("@SAY Mi nombre es MyRobot $", this);
            return;
        }
        if (matches.contains("hola")) {
            Communicator.sendToRobot("@SAY Hola $", this);
            return;
        }
        if (matches.contains("girar derecha") || matches.contains("derecha")) {
            Communicator.sendToRobot("@DRETA$", this);
            return;
        }
        if (matches.contains("girar izquierda") || matches.contains("izquierda")) {
            Communicator.sendToRobot("@ESQUERRA$", this);
            return;
        }
        if (matches.contains("avanzar") ||  matches.contains("avanza") || matches.contains("arranca")) {
            Communicator.sendToRobot("@ARRANCA$", this);
            return;
        }
        if (matches.contains("retroceder") || matches.contains("retrocede") || matches.contains("atras")) {
            Communicator.sendToRobot("@ENRERE$", this);
            return;
        }
        if (matches.contains("parar") || matches.contains("para")) {
            Communicator.sendToRobot("@PARA$", this);
            return;
        }
        if (matches.contains("di no")) {
            Communicator.sendToRobot("@NO_NO$", this);
            return;
        }
        if (matches.contains("sonido") || matches.contains("musica")) {
            Communicator.sendToRobot("@SOUND$", this);
            return;
        }
        if (matches.contains("guapa") || matches.contains("guapo") || matches.contains("silba")  ) {
            Communicator.sendToRobot("@SILBA$", this);
            return;
        }
        if (matches.contains("vuelta") || matches.contains("vuelta derecha")) {
            Communicator.sendToRobot("@VOLTA_D$", this);
            return;
        }
        if (matches.contains("vuelta izquierda")) {
            Communicator.sendToRobot("@VOLTA_E$", this);
            return;
        }
        /*
        Incloure en el cas d'afegir un mòdul Text2Speech, el mòbil faria de cervell amb respostes precomposades o inclòs
        afegir IA
        if (matches.contains("quien es el mejor jugador del mundo") || matches.contains("mejor jugador del mundo")) {
            Communicator.send("El mejor jugador es Mesi", this);
        }
        */
    }

    /**
     * Funció que dona accés a la pantalla de Configuració
     *
     * @param view
     */
    public void goSettings(View view) {
        Intent intent = new Intent(this, SettingsActivity.class);
        startActivity(intent);
    }

    /**
     * Funció que dona accés a la pantalla de Crèdits
     *
     * @param view
     */
    public void goAcercaDe(View view) {
        Intent intent = new Intent(this, AcercaDeActivity.class);
        intent.putExtra("splash", false);
        startActivity(intent);
    }

    /**
     * Funció que dona accés a la pantalla de Splash
     */
    public void splash() {
        Intent intent = new Intent(this, AcercaDeActivity.class);
        startActivity(intent);
    }

    /**
     * Funció que envia una order al robot quan es prem un botó
     *
     * @param view
     */
    public void send(View view) {
        if (Communicator.robotIP == null || Communicator.robotIP.length() == 0) {
            Toast.makeText(getBaseContext(), "No se ha configurado la IP del robot", Toast.LENGTH_SHORT).show();
        }

        if (R.id.btnUP == view.getId()) {
            Communicator.sendToRobot("@ARRANCA$", this);
            return;
        }
        if (R.id.btnDOWN == view.getId()) {
            Communicator.sendToRobot("@ENRERE$", this);
            return;
        }
        if (R.id.btnLEFT == view.getId()) {
            Communicator.sendToRobot("@ESQUERRA$", this);
            return;
        }
        if (R.id.btnRIGHT == view.getId()) {
            Communicator.sendToRobot("@DRETA$", this);
            return;
        }
        if (R.id.btnSTOP == view.getId()) {
            Communicator.sendToRobot("@PARA$", this);
            return;
        }
        if (R.id.btnNO == view.getId()) {
            Communicator.sendToRobot("@NO_NO$", this);
            return;
        }
        if (R.id.btnSOUND == view.getId()) {
            Communicator.sendToRobot("@SOUND$", this);
            return;
        }
        if (R.id.btnLeftLoop == view.getId()) {
            Communicator.sendToRobot("@VOLTA_E$", this);
            return;
        }
        if (R.id.btnRightLoop == view.getId()) {
            Communicator.sendToRobot("@VOLTA_D$", this);
            return;
        }
    }

    @Override
    public void onResponseRequest(boolean result, final String text) {
        if (!result) {
            runOnUiThread(new Runnable() {
                public void run() {
                    Toast.makeText(getBaseContext(), text, Toast.LENGTH_SHORT).show();
                }
            });
        }
    }
}