package myrobot.main.comm;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.Socket;

/**
 * Classe que implementa la comunicació per sockets amb el robot
 * El mòbil haurà de estar a la mateixa xarxa que el robot
 */
public class Communicator {
    public static String robotIP = null;

    // Funció per enviar sense rebre cap resposta
   /* public static void send(final String text) {
        send(text, null);
    }*/
    public static void sendToRobot(String text, final Callback call) {
        send(robotIP, text, call);
    }

    public static void send(final String ip, final String text, final Callback call) {
        // Android exigeix fer les crides asíncrones per la qual cosa, s'implementa en un fil que
        // no bloqueixi el fil principal
        Thread client = new Thread() {
            @Override
            public void run() {
                Socket socket = null;
                BufferedReader in = null;
                OutputStream os = null;

                try {
                    socket = new Socket(ip, 80);
                    socket.setSoTimeout(5000);

                    os = socket.getOutputStream();
                    os.write(text.getBytes());
                    os.flush();

                    InputStream input = socket.getInputStream();
                    BufferedReader reader = new BufferedReader(new InputStreamReader(input));
                    String line = reader.readLine();    // reads a line of text
                    if (call != null) call.onResponseRequest(true, line);

                    Thread.sleep(100);
                } catch (Exception ex) {
                    call.onResponseRequest(false, ex.getMessage());
                } finally {
                    if (os != null) {
                        try {
                            os.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                    if (in != null) {
                        try {
                            in.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                    if (socket != null) {
                        try {
                            socket.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                }
            }
        };

        client.start();

    }


}
